from infisical.models.api import GetSecretDTO, SecretResponse
from requests import Session


def get_secret_req(api_request: Session, options: GetSecretDTO) -> SecretResponse:
    response = api_request.get(
        url=f"/api/v3/secrets/{options.secret_name}",
        params={
            "workspaceId": options.workspace_id,
            "environment": options.environment,
            "type": options.type,
            "secretPath": options.path,
        },
    )

    return SecretResponse.parse_obj(response.json())



from infisical.models.api import GetSecretsDTO, SecretsResponse
from requests import Session


def get_secrets_req(api_request: Session, options: GetSecretsDTO) -> SecretsResponse:
    """Send request again Infisical API to fetch secrets.
    See more information on https://infisical.com/docs/api-reference/endpoints/secrets/read

    :param api_request: The :class:`requests.Session` instance used to perform the request
    :param workspace_id: The ID of the workspace
    :param environment: The environment
    :return: Returns the API response as-is
    """
    response = api_request.get(
        "/api/v3/secrets",
        params={
            "environment": options.environment,
            "workspaceId": options.workspace_id,
        },
    )

    return SecretsResponse.parse_obj(response.json())



from typing import Dict, Optional

from pydantic import BaseModel
from requests import Session
from typing_extensions import Literal


class WorkspaceConfig(BaseModel):
    workspace_id: str
    workspace_key: str


class ClientConfig(BaseModel):
    auth_mode: Literal["service_token"]
    credentials: Dict[Literal["service_token_key"], str]
    workspace_config: Optional[WorkspaceConfig]
    cache_ttl: int



import os

import pytest
from infisical import InfisicalClient


@pytest.fixture(scope="module")
def client():
    infisical_client = InfisicalClient(
        token=os.environ["INFISICAL_TOKEN"], site_url=os.environ["SITE_URL"], debug=True
    )

    infisical_client.create_secret("KEY_ONE", "KEY_ONE_VAL")
    infisical_client.create_secret("KEY_ONE", "KEY_ONE_VAL_PERSONAL", type="personal")
    infisical_client.create_secret("KEY_TWO", "KEY_TWO_VAL")

    # infisical_client.create_secret("TESTOO", "sssss", path="/foo", environment="dev")

    yield infisical_client

    infisical_client.delete_secret("KEY_ONE")
    infisical_client.delete_secret("KEY_TWO")
    infisical_client.delete_secret("KEY_THREE")


def test_get_overriden_personal_secret(client: InfisicalClient):
    secret = client.get_secret("KEY_ONE")
    assert secret.secret_name == "KEY_ONE"
    assert secret.secret_value == "KEY_ONE_VAL_PERSONAL"
    assert secret.type == "personal"


def test_get_shared_secret_specified(client: InfisicalClient):
    secret = client.get_secret("KEY_ONE", type="shared")
    assert secret.secret_name == "KEY_ONE"
    assert secret.secret_value == "KEY_ONE_VAL"
    assert secret.type == "shared"


def test_get_shared_secret(client: InfisicalClient):
    secret = client.get_secret("KEY_TWO")
    assert secret.secret_name == "KEY_TWO"
    assert secret.secret_value == "KEY_TWO_VAL"
    assert secret.type == "shared"


def test_create_shared_secret(client: InfisicalClient):
    secret = client.create_secret("KEY_THREE", "KEY_THREE_VAL")
    assert secret.secret_name == "KEY_THREE"
    assert secret.secret_value == "KEY_THREE_VAL"
    assert secret.type == "shared"


def test_create_personal_secret(client: InfisicalClient):
    client.create_secret("KEY_FOUR", "KEY_FOUR_VAL")
    personal_secret = client.create_secret(
        "KEY_FOUR", "KEY_FOUR_VAL_PERSONAL", type="personal"
    )

    assert personal_secret.secret_name == "KEY_FOUR"
    assert personal_secret.secret_value == "KEY_FOUR_VAL_PERSONAL"
    assert personal_secret.type == "personal"


def test_update_shared_secret(client: InfisicalClient):
    secret = client.update_secret("KEY_THREE", "FOO")

    assert secret.secret_name == "KEY_THREE"
    assert secret.secret_value == "FOO"
    assert secret.type == "shared"


def test_update_personal_secret(client: InfisicalClient):
    secret = client.update_secret("KEY_FOUR", "BAR", type="personal")
    assert secret.secret_name == "KEY_FOUR"
    assert secret.secret_value == "BAR"
    assert secret.type == "personal"


def test_delete_personal_secret(client: InfisicalClient):
    secret = client.delete_secret("KEY_FOUR", type="personal")
    assert secret.secret_name == "KEY_FOUR"
    assert secret.secret_value == "BAR"
    assert secret.type == "personal"


def test_delete_shared_secret(client: InfisicalClient):
    secret = client.delete_secret("KEY_FOUR")
    assert secret.secret_name == "KEY_FOUR"
    assert secret.secret_value == "KEY_FOUR_VAL"
    assert secret.type == "shared"


def test_encrypt_decrypt_symmetric(client: InfisicalClient):
    plaintext = "The quick brown fox jumps over the lazy dog"
    key = client.create_symmetric_key()

    ciphertext, iv, tag = client.encrypt_symmetric(plaintext, key)
    cleartext = client.decrypt_symmetric(ciphertext, key, iv, tag)
    assert plaintext == cleartext



from typing import Dict, Optional

from infisical.api import create_api_request_with_auth
from infisical.constants import (
    AUTH_MODE_SERVICE_TOKEN,
    INFISICAL_URL,
    SERVICE_TOKEN_REGEX,
)
from infisical.exceptions import InfisicalTokenError
from infisical.helpers.client import (
    create_secret_helper,
    delete_secret_helper,
    get_all_secrets_helper,
    get_secret_helper,
    update_secret_helper,
)
from infisical.models.models import SecretBundle
from infisical.models.secret_service import ClientConfig
from infisical.utils.crypto import (
    create_symmetric_key_helper,
    decrypt_symmetric_helper,
    encrypt_symmetric_helper,
)
from typing_extensions import Literal


class InfisicalClient:
    def __init__(
        self,
        token: Optional[str] = None,
        site_url: str = INFISICAL_URL,
        debug: bool = False,
        cache_ttl: int = 300,
    ):
        self.cache: Dict[str, SecretBundle] = {}
        self.client_config: Optional[ClientConfig] = None

        if token and token != "":
            token_match = SERVICE_TOKEN_REGEX.fullmatch(token)

            if token_match is None:
                raise InfisicalTokenError("The token is not in correct format!")

            service_token = token_match.group(1)
            service_token_key = token_match.group(2)

            self.client_config = ClientConfig(
                auth_mode=AUTH_MODE_SERVICE_TOKEN,
                credentials={"service_token_key": service_token_key},
                cache_ttl=cache_ttl,
            )

            self.api_request = create_api_request_with_auth(site_url, service_token)

        self.debug = debug

    def get_all_secrets(self, environment: str = "dev", path: str = "/"):
        """Return all the secrets accessible by the instance of Infisical"""
        return get_all_secrets_helper(self, environment, path)

    def get_secret(
        self,
        secret_name: str,
        type: Literal["shared", "personal"] = "personal",
        environment: str = "dev",
        path: str = "/",
    ) -> SecretBundle:
        """Return secret with name `secret_name`

        :param secret_name: Key of secret
        :param type: Type of secret that is either "shared" or "personal"
        :return: Secret bundle for secret with name `secret_name`
        """
        return get_secret_helper(self, secret_name, type, environment, path)

    def create_secret(
        self,
        secret_name: str,
        secret_value: str,
        type: Literal["shared", "personal"] = "shared",
        environment: str = "dev",
        path: str = "/",
    ) -> SecretBundle:
        """Create secret with name `secret_name` and value `secret_value`

        :param secret_name: Name of secret to create
        :param secret_value: Value of secret to create
        :param type: Type of secret to create that is either "shared" or "personal"
        :return: Secret bundle for created secret with name `secret_name`
        """
        return create_secret_helper(
            self, secret_name, secret_value, type, environment, path
        )

    def update_secret(
        self,
        secret_name: str,
        secret_value: str,
        type: Literal["shared", "personal"] = "shared",
        environment: str = "dev",
        path: str = "/",
    ) -> SecretBundle:
        """Update secret with name `secret_name` and value `secret_value`

        :param secret_name: Name of secret to update
        :param secret_value: New value of secret to update
        :param type: Type of secret to update that is either "shared" or "personal"
        :return: Secret bundle for updated secret with name `secret_name`
        """
        return update_secret_helper(
            self, secret_name, secret_value, type, environment, path
        )

    def delete_secret(
        self,
        secret_name: str,
        type: Literal["shared", "personal"] = "shared",
        environment: str = "dev",
        path: str = "/",
    ):
        """Delete secret with name `secret_name`

        :param secret_name: Name of secret to delete
        :param type: Type of secret to update that is either "shared" or "personal"
        :return: Secret bundle for updated secret with name `secret_name`
        """
        return delete_secret_helper(self, secret_name, type, environment, path)

    def create_symmetric_key(self) -> str:
        """Create a base64-encoded, 256-bit symmetric key"""
        return create_symmetric_key_helper()

    def encrypt_symmetric(self, plaintext: str, key: str):
        """Encrypt the plaintext `plaintext` with the (base64) 256-bit secret key `key`"""
        return encrypt_symmetric_helper(plaintext, key)

    def decrypt_symmetric(self, ciphertext: str, key: str, iv: str, tag: str):
        """Decrypt the ciphertext `ciphertext` with the (base64) 256-bit secret key `key`, provided `iv` and `tag`"""
        return decrypt_symmetric_helper(ciphertext, key, iv, tag)



from infisical.__version__ import __version__
from infisical.utils.http import BaseUrlSession, get_http_client

USER_AGENT = f"InfisicalPythonSDK/{__version__}"


def create_api_request_with_auth(base_url: str, service_token: str) -> BaseUrlSession:
    """Returns a :class:`requests.Session` with a ``base_url`` and the authorization
    bearer set to the ``service_token``.

    :param base_url: The base url to use
    :param service_token: The service token to use as a authorization bearer token
    :return: A :class:`requests.Session` instance preconfigured
    """
    api_request = get_http_client(base_url=base_url.rstrip("/"))

    api_request.headers.update({"User-Agent": USER_AGENT})
    api_request.headers.update({"Content-Type": "application/json"})
    api_request.headers.update({"Authorization": f"Bearer {service_token}"})

    return api_request



from datetime import datetime
from typing import List, Optional

from pydantic import BaseModel, Field


class GetServiceTokenDetailsUserResponse(BaseModel):
    id: str = Field(..., alias="_id")
    email: str
    created_at: datetime = Field(..., alias="createdAt")
    updated_at: datetime = Field(..., alias="updatedAt")
    v: int = Field(..., alias="__v")
    first_name: str = Field(..., alias="firstName")
    last_name: str = Field(..., alias="lastName")


class GetServiceTokenDetailsResponse(BaseModel):
    id: str = Field(..., alias="_id")
    name: str
    workspace: str
    user: GetServiceTokenDetailsUserResponse
    expires_at: Optional[datetime] = Field(None, alias="expiresAt")
    encrypted_key: str = Field(..., alias="encryptedKey")
    iv: str
    tag: str
    created_at: datetime = Field(..., alias="createdAt")
    updated_at: datetime = Field(..., alias="updatedAt")
    v: int = Field(..., alias="__v")



from datetime import datetime
from typing import Optional

from pydantic import BaseModel, Field
from typing_extensions import Literal


class Secret(BaseModel):
    id: str = Field(..., alias="_id")
    version: int
    workspace: str
    user: Optional[str]
    type: Literal["shared", "personal"]
    environment: str
    secret_key_ciphertext: str = Field(..., alias="secretKeyCiphertext")
    secret_key_iv: str = Field(..., alias="secretKeyIV")
    secret_key_tag: str = Field(..., alias="secretKeyTag")
    secret_value_ciphertext: str = Field(..., alias="secretValueCiphertext")
    secret_value_iv: str = Field(..., alias="secretValueIV")
    secret_value_tag: str = Field(..., alias="secretValueTag")
    created_at: datetime = Field(..., alias="createdAt")
    updated_at: datetime = Field(..., alias="updatedAt")


class SecretBundle(BaseModel):
    secret_name: str
    secret_value: Optional[str]
    version: Optional[int]
    workspace: Optional[str]
    environment: Optional[str]
    type: Optional[Literal["shared", "personal"]]
    created_at: Optional[datetime] = Field(None, alias="createdAt")
    updated_at: Optional[datetime] = Field(None, alias="updatedAt")
    is_fallback: bool
    last_fetched_at: datetime


class ServiceTokenData(BaseModel):
    id: str = Field(..., alias="_id")
    name: str
    workspace: str
    environment: str
    user: str
    service_account: str
    last_used: datetime = Field()
    expires_at: datetime = Field(..., alias="expiresAt")
    encrypted_key: str = Field(..., alias="encryptedKey")
    iv: str
    tag: str
    created_at: datetime = Field(..., alias="createdAt")
    updated_at: datetime = Field(..., alias="updatedAt")



from base64 import b64decode, b64encode
from typing import Tuple, Union

from Cryptodome.Cipher import AES
from Cryptodome.Random import get_random_bytes
from nacl import public, utils

Base64String = str
Buffer = Union[bytes, bytearray, memoryview]
import binascii


def encrypt_asymmetric(
    plaintext: Union[Buffer, str],
    public_key: Union[Buffer, Base64String, public.PublicKey],
    private_key: Union[Buffer, Base64String, public.PrivateKey],
) -> Tuple[Base64String, Base64String]:
    """Performs asymmetric encryption of the ``plaintext`` with x25519-xsalsa20-poly1305
    algorithm with the given parameters. Each of those params should be either the raw value in bytes
    or a base64 string.

    :param plaintext: The text to encrypt
    :param public_key: The public key
    :param private_key: The private key
    :raises ValueError: If ``plaintext``, ``public_key`` or ``private_key`` are empty
    :return: A tuple containing the ciphered text and the random nonce used for encryption
    """
    if (not isinstance(public_key, public.PublicKey) and len(public_key) == 0) or (
        not isinstance(private_key, public.PrivateKey) and len(private_key) == 0
    ):
        raise ValueError("Public key and private key cannot be empty!")

    m_plaintext = (
        str.encode(plaintext, "utf-8") if isinstance(plaintext, str) else plaintext
    )
    m_public_key = (
        b64decode(public_key) if isinstance(public_key, Base64String) else public_key
    )
    m_public_key = (
        public.PublicKey(m_public_key)
        if isinstance(m_public_key, (bytes, bytearray, memoryview))
        else m_public_key
    )
    m_private_key = (
        b64decode(private_key) if isinstance(private_key, Base64String) else private_key
    )
    m_private_key = (
        public.PrivateKey(m_private_key)
        if isinstance(m_private_key, (bytes, bytearray, memoryview))
        else m_private_key
    )

    nonce = utils.random(24)
    box = public.Box(m_private_key, m_public_key)
    ciphertext = box.encrypt(m_plaintext, nonce).ciphertext

    return (b64encode(ciphertext).decode("utf-8"), b64encode(nonce).decode("utf-8"))


def decrypt_asymmetric(
    ciphertext: Union[Buffer, Base64String],
    nonce: Union[Buffer, Base64String],
    public_key: Union[Buffer, Base64String, public.PublicKey],
    private_key: Union[Buffer, Base64String, public.PrivateKey],
) -> str:
    """Performs asymmetric decryption of the ``ciphertext`` with x25519-xsalsa20-poly1305
    algorithm with the given parameters. Each of those params should be either the raw value in bytes
    or a base64 string.

    :param ciphertext: The ciphered text to decrypt
    :param nonce: The nonce used for encryption
    :param public_key: The public key
    :param private_key: The private key
    :raises ValueError: If ``ciphertext``, ``nonce``, ``public_key`` or ``private_key`` are empty
    :return: The deciphered text
    """
    if (
        len(ciphertext) == 0
        or len(nonce) == 0
        or (not isinstance(public_key, public.PublicKey) and len(public_key) == 0)
        or (not isinstance(private_key, public.PrivateKey) and len(private_key) == 0)
    ):
        raise ValueError(
            "Public key, private key, ciphertext and nonce cannot be empty!"
        )

    m_ciphertext = (
        b64decode(ciphertext) if isinstance(ciphertext, Base64String) else ciphertext
    )
    m_nonce = b64decode(nonce) if isinstance(nonce, Base64String) else nonce
    m_public_key = (
        b64decode(public_key) if isinstance(public_key, Base64String) else public_key
    )
    m_public_key = (
        public.PublicKey(m_public_key)
        if isinstance(m_public_key, (bytes, bytearray, memoryview))
        else m_public_key
    )
    m_private_key = (
        b64decode(private_key) if isinstance(private_key, Base64String) else private_key
    )
    m_private_key = (
        public.PrivateKey(m_private_key)
        if isinstance(m_private_key, (bytes, bytearray, memoryview))
        else m_private_key
    )

    box = public.Box(m_private_key, m_public_key)
    plaintext = box.decrypt(m_ciphertext, m_nonce)

    return plaintext.decode("utf-8")


def create_symmetric_key_helper():
    return b64encode(get_random_bytes(32)).decode("utf-8")


def encrypt_symmetric_helper(plaintext: str, key: str):
    IV_BYTES_SIZE = 12
    iv = get_random_bytes(12)

    cipher = AES.new(b64decode(key), AES.MODE_GCM, nonce=iv)

    ciphertext, tag = cipher.encrypt_and_digest(plaintext.encode("utf-8"))

    return (
        b64encode(ciphertext).decode("utf-8"),
        b64encode(iv).decode("utf-8"),
        b64encode(tag).decode("utf-8"),
    )


def decrypt_symmetric_helper(ciphertext: str, key: str, iv: str, tag: str):
    cipher = AES.new(b64decode(key), AES.MODE_GCM, nonce=b64decode(iv))
    plaintext = cipher.decrypt_and_verify(b64decode(ciphertext), b64decode(tag))

    return plaintext.decode("utf-8")


def encrypt_symmetric_128_bit_hex_key_utf8(
    plaintext: str, key: str
) -> Tuple[Base64String, Base64String, Base64String]:
    """Encrypts the ``plaintext`` with aes-256-gcm using the given ``key``.
    The key should be either the raw value in bytes or a base64 string.

    :param plaintext: text to encrypt
    :param key: UTF-8, 128-bit AES key used for encryption
    :raises ValueError: If either ``plaintext`` or ``key`` is empty
    :return: Ciphered text
    """
    if len(key) == 0:
        raise ValueError("The given key is empty!")

    BLOCK_SIZE_BYTES = 16

    iv = get_random_bytes(BLOCK_SIZE_BYTES)
    cipher = AES.new(bytes(key, "utf-8"), AES.MODE_GCM, nonce=iv)

    ciphertext, tag = cipher.encrypt_and_digest(str.encode(plaintext, "utf-8"))

    return (
        b64encode(ciphertext).decode("utf-8"),
        b64encode(iv).decode("utf-8"),
        b64encode(tag).decode("utf-8"),
    )


def decrypt_symmetric_128_bit_hex_key_utf8(
    key: str, ciphertext: str, tag: str, iv: str
) -> str:
    """Decrypts the ``ciphertext`` with aes-256-gcm using ``iv``, ``tag``
    and ``key``.

    :param key: UTF-8, 128-bit hex AES key
    :param ciphertext: base64 ciphered text to decrypt
    :param tag: base64 tag/mac used for verification
    :param iv: base64 nonce
    :raises ValueError: If ``ciphertext``, ``iv``, ``tag`` or ``key`` are empty or tag/mac does not match
    :return: Deciphered text
    """
    if len(tag) == 0 or len(iv) == 0 or len(key) == 0:
        raise ValueError("One of the given parameter is empty!")

    try:
        key = bytes(key, "utf-8")
        iv = b64decode(iv)
        tag = b64decode(tag)
        ciphertext = b64decode(ciphertext)

        cipher = AES.new(key, AES.MODE_GCM, nonce=iv)
        plaintext = cipher.decrypt_and_verify(ciphertext, tag)

        return plaintext.decode("utf-8")
    except ValueError:
        raise ValueError("Incorrect decryption or MAC check failed")



from infisical.api.models import GetServiceTokenDetailsResponse
from requests import Session


def get_service_token_data_req(
    api_request: Session,
) -> GetServiceTokenDetailsResponse:
    """Send request again Infisical API to fetch service token data.
    See more information on https://infisical.com/docs/api-reference/endpoints/service-tokens/get

    :param api_request: The :class:`requests.Session` instance used to perform the request
    :return: Returns the API response as-is
    """
    response = api_request.get("/api/v2/service-token")

    return GetServiceTokenDetailsResponse.parse_obj(response.json())
